@@ -20,14 +20,15 @@
 
 #include "application.h"
 
+#include <glib/gi18n.h>
+#include <stdlib.h>
+
 #include "about.h"
 #include "charset.h"
 #include "easytag.h"
 #include "log.h"
 #include "misc.h"
-
-#include <glib/gi18n.h>
-#include <stdlib.h>
+#include "setting.h"
 
 /* TODO: Use G_DEFINE_TYPE_WITH_PRIVATE. */
 G_DEFINE_TYPE (EtApplication, et_application, GTK_TYPE_APPLICATION)
@@ -110,7 +111,7 @@ on_idle_init (EtApplication *self)
     priv = et_application_get_instance_private (self);
 
     ET_Core_Free ();
-    ET_Core_Initialize ();
+    ET_Core_Create ();
 
     if (g_settings_get_boolean (MainSettings, "scan-startup"))
     {
