@@ -20,13 +20,13 @@
 
 #include "application_window.h"
 
-#include <gdk/gdkkeysyms.h>
 #include <glib/gi18n.h>
 
 #include "browser.h"
 #include "cddb_dialog.h"
 #include "easytag.h"
 #include "file_area.h"
+#include "file_list.h"
 #include "load_files_dialog.h"
 #include "log.h"
 #include "misc.h"
@@ -288,8 +288,8 @@ delete_file (ET_File *ETFile, gboolean multiple_files, GError **error)
 {
     GtkWidget *msgdialog;
     GtkWidget *msgdialog_check_button = NULL;
-    gchar *cur_filename;
-    gchar *cur_filename_utf8;
+    const gchar *cur_filename;
+    const gchar *cur_filename_utf8;
     gchar *basename_utf8;
     gint response;
     gint stop_loop;
@@ -1482,7 +1482,7 @@ run_audio_player_using_directory (GError **error)
     for (l = g_list_first (ETCore->ETFileList); l != NULL; l = g_list_next (l))
     {
         ET_File *etfile = (ET_File *)l->data;
-        gchar *path = ((File_Name *)etfile->FileNameCur->data)->value;
+        const gchar *path = ((File_Name *)etfile->FileNameCur->data)->value;
         file_list = g_list_prepend (file_list, g_file_new_for_path (path));
     }
 
@@ -1592,7 +1592,10 @@ et_application_window_dispose (GObject *object)
 
     save_state (self);
 
-    ET_Core_Destroy ();
+    if (ETCore)
+    {
+        ET_Core_Free ();
+    }
 
     if (priv->cddb_dialog)
     {
