@@ -25,9 +25,6 @@
 #include "charset.h"
 
 #include <stdlib.h>
-#include <glib.h>
-#include <string.h>
-#include <errno.h>
 #include <glib/gi18n.h>
 
 #ifdef HAVE_LANGINFO_CODESET
