@@ -17,46 +17,30 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-#include "config.h" // For definition of ENABLE_OGG
-#include <gtk/gtk.h>
-#include <stdio.h>
-#include <string.h>
-#include <gdk/gdkkeysyms.h>
+#include "config.h"
+
+#include "easytag.h"
+
 #include <glib/gi18n.h>
 #include <stdlib.h>
 #include <sys/stat.h>
 #include <unistd.h>
-#include <errno.h>
-#ifdef ENABLE_MP3
-#include <id3tag.h>
-#endif
-#if defined ENABLE_MP3 && defined ENABLE_ID3LIB
-#include <id3.h>
-#endif
 #include <sys/types.h>
 
-#include "easytag.h"
 #include "application_window.h"
 #include "browser.h"
+#include "file_description.h"
+#include "file_list.h"
 #include "log.h"
 #include "misc.h"
 #include "cddb_dialog.h"
-#include "preferences_dialog.h"
 #include "setting.h"
 #include "scan_dialog.h"
-#include "mpeg_header.h"
-#include "id3_tag.h"
-#include "ogg_tag.h"
 #include "et_core.h"
-#include "picture.h"
 #include "charset.h"
 
 #include "win32/win32dep.h"
 
-
-/****************
- * Declarations *
- ****************/
 static GtkWidget *QuitRecursionWindow = NULL;
 
 /* Used to force to hide the msgbox when saving tag */
@@ -68,14 +52,6 @@ static gboolean SF_HideMsgbox_Rename_File;
 /* To remember which button was pressed when renaming file */
 static gint SF_ButtonPressed_Rename_File;
 
-#ifdef ENABLE_FLAC
-    #include <FLAC/metadata.h>
-#endif
-
-
-/**************
- * Prototypes *
- **************/
 static gboolean Write_File_Tag (ET_File *ETFile, gboolean hide_msgbox);
 static gint Save_File (ET_File *ETFile, gboolean multiple_files,
                        gboolean force_saving_files);
@@ -201,11 +177,11 @@ Save_List_Of_Files (GList *etfilelist, gboolean force_saving_files)
         GFile *file;
         GFileInfo *fileinfo;
 
-        ET_File *ETFile = (ET_File *)l->data;
-        File_Tag  *FileTag  = (File_Tag *)ETFile->FileTag->data;
-        File_Name *FileName = (File_Name *)ETFile->FileNameNew->data;
-        gchar *filename_cur = ((File_Name *)ETFile->FileNameCur->data)->value;
-        gchar *filename_cur_utf8 = ((File_Name *)ETFile->FileNameCur->data)->value_utf8;
+        const ET_File *ETFile = (ET_File *)l->data;
+        const File_Tag  *FileTag  = (File_Tag *)ETFile->FileTag->data;
+        const File_Name *FileName = (File_Name *)ETFile->FileNameNew->data;
+        const gchar *filename_cur = ((File_Name *)ETFile->FileNameCur->data)->value;
+        const gchar *filename_cur_utf8 = ((File_Name *)ETFile->FileNameCur->data)->value_utf8;
         gchar *basename_cur_utf8 = g_path_get_basename(filename_cur_utf8);
 
         // Count only the changed files or all files if force_saving_files==TRUE
@@ -402,13 +378,11 @@ static gint
 Save_File (ET_File *ETFile, gboolean multiple_files,
            gboolean force_saving_files)
 {
-    File_Tag  *FileTag;
-    File_Name *FileNameNew;
+    const File_Tag *FileTag;
+    const File_Name *FileNameNew;
     gint stop_loop = 0;
-    //struct stat   statbuf;
-    //gchar *filename_cur = ((File_Name *)ETFile->FileNameCur->data)->value;
-    gchar *filename_cur_utf8 = ((File_Name *)ETFile->FileNameCur->data)->value_utf8;
-    gchar *filename_new_utf8 = ((File_Name *)ETFile->FileNameNew->data)->value_utf8;
+    const gchar *filename_cur_utf8 = ((File_Name *)ETFile->FileNameCur->data)->value_utf8;
+    const gchar *filename_new_utf8 = ((File_Name *)ETFile->FileNameNew->data)->value_utf8;
     gchar *basename_cur_utf8, *basename_new_utf8;
     gchar *dirname_cur_utf8, *dirname_new_utf8;
 
@@ -658,8 +632,8 @@ Save_File (ET_File *ETFile, gboolean multiple_files,
             {
                 gboolean rc;
                 GError *error = NULL;
-                gchar *cur_filename = ((File_Name *)ETFile->FileNameCur->data)->value;
-                gchar *new_filename = ((File_Name *)ETFile->FileNameNew->data)->value;
+                const gchar *cur_filename = ((File_Name *)ETFile->FileNameCur->data)->value;
+                const gchar *new_filename = ((File_Name *)ETFile->FileNameNew->data)->value;
                 rc = et_rename_file (cur_filename, new_filename, &error);
 
                 // if 'SF_HideMsgbox_Rename_File is TRUE', then errors are displayed only in log
@@ -769,12 +743,10 @@ et_rename_file (const char *old_filepath, const char *new_filepath,
         /* Ignore an error if the directory already exists. */
         if (!g_error_matches (*error, G_IO_ERROR, G_IO_ERROR_EXISTS))
         {
-            g_object_unref (file_old);
-            g_object_unref (file_new);
             g_object_unref (file_new_parent);
-            g_assert (error == NULL || *error != NULL);
-            return FALSE;
+            goto err;
         }
+
         g_clear_error (error);
     }
 
@@ -858,7 +830,6 @@ out:
     return TRUE;
 
 err:
-    g_object_unref (file_new_parent);
     g_object_unref (file_old);
     g_object_unref (file_new);
     g_assert (error == NULL || *error != NULL);
@@ -874,7 +845,7 @@ static gboolean
 Write_File_Tag (ET_File *ETFile, gboolean hide_msgbox)
 {
     GError *error = NULL;
-    gchar *cur_filename_utf8 = ((File_Name *)ETFile->FileNameCur->data)->value_utf8;
+    const gchar *cur_filename_utf8 = ((File_Name *)ETFile->FileNameCur->data)->value_utf8;
     gchar *msg = NULL;
     gchar *basename_utf8;
     GtkWidget *msgdialog;
@@ -949,8 +920,8 @@ Read_Directory (const gchar *path_real)
     ReadingDirectory = TRUE;    /* A flag to avoid to start another reading */
 
     /* Initialize file list */
-    ET_Core_Free();
-    ET_Core_Initialize();
+    ET_Core_Free ();
+    ET_Core_Create ();
     et_application_window_update_actions (ET_APPLICATION_WINDOW (MainWindow));
 
     window = ET_APPLICATION_WINDOW (MainWindow);
@@ -1192,7 +1163,7 @@ read_directory_recursively (GList *file_list, GFileEnumerator *dir_enumerator,
                 }
             }
             else if (type == G_FILE_TYPE_REGULAR &&
-                      ET_File_Is_Supported (file_name))
+                     et_file_is_supported (file_name))
             {
                 GFile *file = g_file_get_child (g_file_enumerator_get_container (dir_enumerator),
                                                 file_name);
