@@ -25,6 +25,7 @@
 #include "charset.h"
 #include "et_core.h"
 #include "log.h"
+#include "setting.h"
 #include "tag_area.h"
 
 /* TODO: Use G_DEFINE_TYPE_WITH_PRIVATE. */
@@ -242,7 +243,7 @@ et_file_area_set_file_fields (EtFileArea *self,
     EtFileAreaPrivate *priv;
     GFile *file;
     gchar *text;
-    gchar *cur_filename;
+    const gchar *cur_filename;
     gchar *basename_utf8;
     gchar *pos;
     GFileInfo *info;
