@@ -17,20 +17,13 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-#include <config.h>
-#include <gtk/gtk.h>
-#include <gdk/gdkkeysyms.h>
-#include <gdk/gdk.h>
-#include <glib/gi18n.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <string.h>
-#include <stdio.h>
-#include <unistd.h>
-#include <errno.h>
+#include "config.h"
 
 #include "picture.h"
+
+#include <gtk/gtk.h>
+#include <glib/gi18n.h>
+
 #include "easytag.h"
 #include "log.h"
 #include "misc.h"
@@ -39,13 +32,7 @@
 
 #include "win32/win32dep.h"
 
-
-/**************
- * Prototypes *
- **************/
-
-static const gchar *Picture_Format_String (Picture_Format format);
-
+G_DEFINE_BOXED_TYPE (EtPicture, et_picture, et_picture_copy_single, et_picture_free)
 
 /*
  * Note :
@@ -55,10 +42,6 @@ static const gchar *Picture_Format_String (Picture_Format format);
  *
  */
 
-/*************
- * Functions *
- *************/
-
 /*
  * et_picture_type_from_filename:
  * @filename: UTF-8 representation of a filename
@@ -128,44 +111,35 @@ et_picture_type_from_filename (const gchar *filename_utf8)
 
 /* FIXME: Possibly use gnome_vfs_get_mime_type_for_buffer. */
 Picture_Format
-Picture_Format_From_Data (const Picture *pic)
+Picture_Format_From_Data (const EtPicture *pic)
 {
-    // JPEG : "\xff\xd8"
-    if (pic->data && pic->size > 2
-    &&  pic->data[0] == 0xff
-    &&  pic->data[1] == 0xd8)
-        return PICTURE_FORMAT_JPEG;
+    gsize size;
+    gconstpointer data;
 
-    // PNG : "\x89PNG\x0d\x0a\x1a\x0a"
-    if (pic->data && pic->size > 8
-    &&  pic->data[0] == 0x89
-    &&  pic->data[1] == 0x50
-    &&  pic->data[2] == 0x4e
-    &&  pic->data[3] == 0x47
-    &&  pic->data[4] == 0x0d
-    &&  pic->data[5] == 0x0a
-    &&  pic->data[6] == 0x1a
-    &&  pic->data[7] == 0x0a)
+    g_return_val_if_fail (pic != NULL, PICTURE_FORMAT_UNKNOWN);
+
+    data = g_bytes_get_data (pic->bytes, &size);
+
+    /* JPEG : "\xff\xd8\xff". */
+    if (size > 3 && (memcmp (data, "\xff\xd8\xff", 3) == 0))
+    {
+        return PICTURE_FORMAT_JPEG;
+    }
+
+    /* PNG : "\x89PNG\x0d\x0a\x1a\x0a". */
+    if (size > 8 && (memcmp (data, "\x89PNG\x0d\x0a\x1a\x0a", 8) == 0))
+    {
         return PICTURE_FORMAT_PNG;
+    }
     
     /* GIF: "GIF87a" */
-    if (pic->data && pic->size > 6
-    &&  pic->data[0] == 0x47
-    &&  pic->data[1] == 0x49
-    &&  pic->data[2] == 0x46
-    &&  pic->data[3] == 0x38
-    &&  pic->data[4] == 0x37
-    &&  pic->data[5] == 0x61)
+    if (size > 6 && (memcmp (data, "GIF87a", 6) == 0))
+    {
         return PICTURE_FORMAT_GIF;
+    }
 
     /* GIF: "GIF89a" */
-    if (pic->data && pic->size > 6
-        &&  pic->data[0] == 0x47
-        &&  pic->data[1] == 0x49
-        &&  pic->data[2] == 0x46
-        &&  pic->data[3] == 0x38
-        &&  pic->data[4] == 0x39
-        &&  pic->data[5] == 0x61)
+    if (size > 6 && (memcmp (data, "GIF89a", 6) == 0))
     {
         return PICTURE_FORMAT_GIF;
     }
@@ -173,7 +147,8 @@ Picture_Format_From_Data (const Picture *pic)
     return PICTURE_FORMAT_UNKNOWN;
 }
 
-const gchar *Picture_Mime_Type_String (Picture_Format format)
+const gchar *
+Picture_Mime_Type_String (Picture_Format format)
 {
     switch (format)
     {
@@ -261,8 +236,8 @@ Picture_Type_String (EtPictureType type)
 }
 
 gchar *
-Picture_Info (const Picture *pic,
-              ET_Tag_Type tag_type)
+et_picture_format_info (const EtPicture *pic,
+                        ET_Tag_Type tag_type)
 {
     const gchar *format, *desc, *type;
     gchar *r, *size_str;
@@ -274,8 +249,8 @@ Picture_Info (const Picture *pic,
     else
         desc = "";
 
-    type = Picture_Type_String(pic->type);
-    size_str = g_format_size (pic->size);
+    type = Picture_Type_String (pic->type);
+    size_str = g_format_size (g_bytes_get_size (pic->bytes));
 
     // Behaviour following the tag type...
     switch (tag_type)
@@ -304,52 +279,87 @@ Picture_Info (const Picture *pic,
     return r;
 }
 
-Picture *Picture_Allocate (void)
+/*
+ * et_picture_new:
+ * @type: the image type
+ * @description: a text description
+ * @width: image width
+ * @height image height
+ * @bytes: image data
+ *
+ * Create a new #EtPicture instance, copying the string and adding a reference
+ * to the image data.
+ *
+ * Returns: a new #EtPicture, or %NULL on failure
+ */
+EtPicture *
+et_picture_new (EtPictureType type,
+                const gchar *description,
+                guint width,
+                guint height,
+                GBytes *bytes)
 {
-    Picture *pic = g_malloc0(sizeof(Picture));
+    EtPicture *pic;
+
+    g_return_val_if_fail (description != NULL, NULL);
+    g_return_val_if_fail (bytes != NULL, NULL);
+
+    pic = g_slice_new (EtPicture);
+
+    pic->type = type;
+    pic->description = g_strdup (description);
+    pic->width = width;
+    pic->height = height;
+    pic->bytes = g_bytes_ref (bytes);
+    pic->next = NULL;
+
     return pic;
 }
 
-Picture *Picture_Copy_One (const Picture *pic)
+EtPicture *
+et_picture_copy_single (const EtPicture *pic)
 {
-    Picture *pic2;
+    EtPicture *pic2;
 
     g_return_val_if_fail (pic != NULL, NULL);
 
-    pic2 = Picture_Allocate();
-    pic2->type = pic->type;
-    pic2->width  = pic->width;
-    pic2->height = pic->height;
-    if (pic->description)
-        pic2->description = g_strdup(pic->description);
-    if (pic->data)
+    pic2 = et_picture_new (pic->type, pic->description, pic->width,
+                           pic->height, pic->bytes);
+
+    return pic2;
+}
+
+EtPicture *
+et_picture_copy_all (const EtPicture *pic)
+{
+    EtPicture *pic2 = et_picture_copy_single (pic);
+
+    if (pic->next)
     {
-        pic2->size = pic->size;
-        pic2->data = g_malloc(pic2->size);
-        memcpy(pic2->data, pic->data, pic->size);
+        pic2->next = et_picture_copy_all (pic->next);
     }
+
     return pic2;
 }
 
-Picture *Picture_Copy (const Picture *pic)
+void
+et_picture_free (EtPicture *pic)
 {
-    Picture *pic2 = Picture_Copy_One(pic);
-    if (pic->next)
-        pic2->next = Picture_Copy(pic->next);
-    return pic2;
-}
-
-void Picture_Free (Picture *pic)
-{
-    if (!pic)
+    if (pic == NULL)
+    {
         return;
+    }
+
     if (pic->next)
-        Picture_Free(pic->next);
-    if (pic->description)
-        g_free(pic->description);
-    if (pic->data)
-        g_free(pic->data);
-    g_free(pic);
+    {
+        et_picture_free (pic->next);
+    }
+
+    g_free (pic->description);
+    g_bytes_unref (pic->bytes);
+    pic->bytes = NULL;
+
+    g_slice_free (EtPicture, pic);
 }
 
 
@@ -360,9 +370,9 @@ void Picture_Free (Picture *pic)
  *
  * Load an image from the supplied @file.
  *
- * Returns: an image on success, %NULL otherwise
+ * Returns: image data on success, %NULL otherwise
  */
-Picture *
+GBytes *
 et_picture_load_file_data (GFile *file, GError **error)
 {
     gsize size;
@@ -415,7 +425,9 @@ et_picture_load_file_data (GFile *file, GError **error)
     else
     {
         /* Image loaded. */
-        Picture *pic;
+        GBytes *bytes;
+        gpointer data;
+        gsize data_size;
 
         g_object_unref (file_istream);
 
@@ -428,19 +440,20 @@ et_picture_load_file_data (GFile *file, GError **error)
 
         g_assert (error == NULL || *error == NULL);
 
-        pic = Picture_Allocate ();
-        pic->data = g_memory_output_stream_steal_data (G_MEMORY_OUTPUT_STREAM (ostream));
-        pic->size = g_memory_output_stream_get_data_size (G_MEMORY_OUTPUT_STREAM (ostream));
+        data = g_memory_output_stream_steal_data (G_MEMORY_OUTPUT_STREAM (ostream));
+        data_size = g_memory_output_stream_get_data_size (G_MEMORY_OUTPUT_STREAM (ostream));
+        bytes = g_bytes_new_take (data, data_size);
+        /* TODO: Use g_memory_output_stream_steal_as_bytes(). */
 
         g_object_unref (ostream);
         g_assert (error == NULL || *error == NULL);
-        return pic;
+        return bytes;
     }
 }
 
 /*
  * et_picture_save_file_data:
- * @pic: the #Picture from which to take an image
+ * @pic: the #EtPicture from which to take an image
  * @file: the #GFile for which to save an image
  * @error: a #GError to provide information on errors, or %NULL to ignore
  *
@@ -449,9 +462,13 @@ et_picture_load_file_data (GFile *file, GError **error)
  * Returns: %TRUE on success, %FALSE otherwise
  */
 gboolean
-et_picture_save_file_data (const Picture *pic, GFile *file, GError **error)
+et_picture_save_file_data (const EtPicture *pic,
+                           GFile *file,
+                           GError **error)
 {
     GFileOutputStream *file_ostream;
+    gconstpointer data;
+    gsize data_size;
     gsize bytes_written;
 
     g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
@@ -465,12 +482,14 @@ et_picture_save_file_data (const Picture *pic, GFile *file, GError **error)
         return FALSE;
     }
 
-    if (!g_output_stream_write_all (G_OUTPUT_STREAM (file_ostream), pic->data,
-                                    pic->size, &bytes_written, NULL, error))
+    data = g_bytes_get_data (pic->bytes, &data_size);
+
+    if (!g_output_stream_write_all (G_OUTPUT_STREAM (file_ostream), data,
+                                    data_size, &bytes_written, NULL, error))
     {
         g_debug ("Only %" G_GSIZE_FORMAT " bytes out of %" G_GSIZE_FORMAT
                  " bytes of picture data were written", bytes_written,
-                 pic->size);
+                 g_bytes_get_size (pic->bytes));
         g_object_unref (file_ostream);
         g_assert (error == NULL || *error != NULL);
         return FALSE;
