@@ -22,13 +22,9 @@
 #include "preferences_dialog.h"
 
 #include <errno.h>
-#include <stdlib.h>
 #include <gdk/gdkkeysyms.h>
 #include <glib/gi18n.h>
-#include <stdio.h>
 #include <string.h>
-#include <sys/stat.h>
-#include <unistd.h>
 
 #include "application_window.h"
 #include "setting.h"
@@ -510,15 +506,15 @@ create_preferences_dialog (EtPreferencesDialog *self)
     /* Disc formatting. */
     pad_disc_number = GTK_WIDGET (gtk_builder_get_object (builder,
                                                           "tags_disc_check"));
-    g_settings_bind (MainSettings, "tag-number-padded", pad_disc_number,
+    g_settings_bind (MainSettings, "tag-disc-padded", pad_disc_number,
                      "active", G_SETTINGS_BIND_DEFAULT);
 
     pad_disc_number_spinbutton = GTK_WIDGET (gtk_builder_get_object (builder,
                                                                      "tags_disc_button"));
-    g_settings_bind (MainSettings, "tag-number-length",
+    g_settings_bind (MainSettings, "tag-disc-length",
                      pad_disc_number_spinbutton, "value",
                      G_SETTINGS_BIND_DEFAULT);
-    g_settings_bind (MainSettings, "tag-number-padded",
+    g_settings_bind (MainSettings, "tag-disc-padded",
                      pad_disc_number_spinbutton, "sensitive",
                      G_SETTINGS_BIND_GET);
     g_signal_emit_by_name (G_OBJECT (pad_disc_number), "toggled");
