@@ -22,16 +22,11 @@
 
 #ifdef ENABLE_FLAC
 
-#include <gtk/gtk.h>
 #include <glib/gi18n.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
 #include <errno.h>
-#include <FLAC/metadata.h>
-#include <unistd.h>
 
 #include "easytag.h"
+#include "flac_private.h"
 #include "flac_tag.h"
 #include "vcedit.h"
 #include "et_core.h"
@@ -41,14 +36,8 @@
 #include "picture.h"
 #include "charset.h"
 
-
-/***************
- * Declaration *
- ***************/
-
 #define MULTIFIELD_SEPARATOR " - "
 
-
 /* FLAC uses Ogg Vorbis comments
  * Ogg Vorbis fields names :
  *  - TITLE        : Track name
@@ -76,23 +65,12 @@
  * of any language.
  */
 
-
-
-/**************
- * Prototypes *
- **************/
-
 static gboolean Flac_Write_Delimetered_Tag (FLAC__StreamMetadata *vc_block, const gchar *tag_name, gchar *values);
 static gboolean Flac_Write_Tag (FLAC__StreamMetadata *vc_block, const gchar *tag_name, gchar *value);
 static gboolean Flac_Set_Tag (FLAC__StreamMetadata *vc_block, const gchar *tag_name, gchar *value, gboolean split);
 
-
-/*************
- * Functions *
- *************/
-
 /*
- * Read tag data from a FLAC file using the level 1 flac interface,
+ * Read tag data from a FLAC file using the level 2 flac interface,
  * Note:
  *  - if field is found but contains no info (strlen(str)==0), we don't read it
  */
@@ -101,60 +79,75 @@ flac_tag_read_file_tag (GFile *file,
                         File_Tag *FileTag,
                         GError **error)
 {
-    FLAC__Metadata_SimpleIterator *iter;
-    const gchar *flac_error_msg;
+    FLAC__Metadata_Chain *chain;
+    EtFlacReadState state;
+    FLAC__IOCallbacks callbacks = { et_flac_read_func,
+                                    NULL, /* Do not set a write callback. */
+                                    et_flac_seek_func, et_flac_tell_func,
+                                    et_flac_eof_func,
+                                    et_flac_read_close_func };
+    FLAC__Metadata_Iterator *iter;
+
     gchar *string = NULL;
-    gchar *filename;
-    gchar *filename_utf8;
     guint i;
-    Picture *prev_pic = NULL;
+    EtPicture *prev_pic = NULL;
     //gint j = 1;
 
     g_return_val_if_fail (file != NULL && FileTag != NULL, FALSE);
     g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
-    /* Initialize the iterator for the blocks. */
-    filename = g_file_get_path (file);
-    iter = FLAC__metadata_simple_iterator_new();
+    chain = FLAC__metadata_chain_new ();
 
-    if ( iter == NULL || !FLAC__metadata_simple_iterator_init(iter, filename, true, false) )
+    if (chain == NULL)
     {
-        if ( iter == NULL )
-        {
-            // Error with "FLAC__metadata_simple_iterator_new"
-            flac_error_msg = FLAC__Metadata_SimpleIteratorStatusString[FLAC__METADATA_SIMPLE_ITERATOR_STATUS_MEMORY_ALLOCATION_ERROR];
-        }else
-        {
-            // Error with "FLAC__metadata_simple_iterator_init"
-            FLAC__Metadata_SimpleIteratorStatus status = FLAC__metadata_simple_iterator_status(iter);
-            flac_error_msg = FLAC__Metadata_SimpleIteratorStatusString[status];
-            
-            FLAC__metadata_simple_iterator_delete(iter);
-        }
-
-        g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
-                     _("Error while opening file: %s"), flac_error_msg);
-        g_free (filename);
+        g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_NOMEM, "%s",
+                     g_strerror (ENOMEM));
         return FALSE;
     }
-    
-    filename_utf8 = filename_to_display (filename);
-    g_free (filename);
 
-    /* libFLAC is able to detect (and skip) ID3v2 tags by itself */
+    state.error = NULL;
+    state.istream = g_file_read (file, NULL, &state.error);
+    state.seekable = G_SEEKABLE (state.istream);
 
-    while (FLAC__metadata_simple_iterator_next(iter))
+    if (!FLAC__metadata_chain_read_with_callbacks (chain, &state, callbacks))
     {
-        // Get block data
-        FLAC__StreamMetadata *block = FLAC__metadata_simple_iterator_get_block(iter);
-        //g_print("Read: %d %s -> block type: %d\n",j++,g_path_get_basename(filename),FLAC__metadata_simple_iterator_get_block_type(iter));
-        
-        // Action to do according the type
-        switch ( FLAC__metadata_simple_iterator_get_block_type(iter) )
+        FLAC__Metadata_ChainStatus status;
+
+        status = FLAC__metadata_chain_status (chain);
+
+        switch (status)
+        {
+            /* TODO: Provide a dedicated error enum corresponding to status. */
+            default:
+                g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED, "%s",
+                             _("Error opening FLAC file"));
+                et_flac_read_close_func (&state);
+                break;
+        }
+
+        return FALSE;
+    }
+
+    iter = FLAC__metadata_iterator_new ();
+
+    if (iter == NULL)
+    {
+        et_flac_read_close_func (&state);
+        g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_NOMEM, "%s",
+                     g_strerror (ENOMEM));
+        return FALSE;
+    }
+
+    FLAC__metadata_iterator_init (iter, chain);
+
+    while (FLAC__metadata_iterator_next (iter))
+    {
+        FLAC__StreamMetadata *block;
+
+        block = FLAC__metadata_iterator_get_block (iter);
+
+        switch (block->type)
         {
-            //
-            // Read the VORBIS_COMMENT block (only one should exist)
-            //
             case FLAC__METADATA_TYPE_VORBIS_COMMENT:
             {
                 FLAC__StreamMetadata_VorbisComment       *vc;
@@ -668,48 +661,41 @@ flac_tag_read_file_tag (GFile *file,
             //
             case FLAC__METADATA_TYPE_PICTURE: 
             {
-                
-                /***********
-                 * Picture *
-                 ***********/
+                /* Picture. */
                 FLAC__StreamMetadata_Picture *p;
-                Picture *pic;
+                GBytes *bytes;
+                EtPicture *pic;
             
-                // Get picture data from block
+                /* Get picture data from block. */
                 p = &block->data.picture;
+
+                bytes = g_bytes_new (p->data, p->data_length);
             
-                pic = Picture_Allocate();
+                pic = et_picture_new (p->type, (const gchar *)p->description,
+                                      0, 0, bytes);
+                g_bytes_unref (bytes);
+
                 if (!prev_pic)
+                {
                     FileTag->picture = pic;
+                }
                 else
+                {
                     prev_pic->next = pic;
+                }
+
                 prev_pic = pic;
-
-                pic->size = p->data_length;
-                pic->data = g_memdup(p->data,pic->size);
-                pic->type = p->type;
-                pic->description = g_strdup((gchar *)p->description);
-                // Not necessary: will be calculated later
-                //pic->height = p->height;
-                //pic->width  = p->width;
-
-                //g_print("Image type : %s\n",FLAC__StreamMetadata_Picture_TypeString[p->type]);
-                //g_print("Mime    type : %s\n",p->mime_type);
-
                 break;
             }
                 
             default:
                 break;
         }
-        
-        /* Free block data. */
-        FLAC__metadata_object_delete (block);
     }
-    
-    // Free iter
-    FLAC__metadata_simple_iterator_delete(iter);
 
+    FLAC__metadata_iterator_delete (iter);
+    FLAC__metadata_chain_delete (chain);
+    et_flac_read_close_func (&state);
 
 #ifdef ENABLE_MP3
     /* If no FLAC vorbis tag found : we try to get the ID3 tag if it exists
@@ -732,8 +718,9 @@ flac_tag_read_file_tag (GFile *file,
       && FileTag->encoded_by  == NULL
       && FileTag->picture     == NULL)
     {
-        /* Ignore errors. */
-        id3tag_read_file_tag (file, FileTag, NULL);
+        gboolean rc;
+
+        rc = id3tag_read_file_tag (file, FileTag, NULL);
 
         // If an ID3 tag has been found (and no FLAC tag), we mark the file as
         // unsaved to rewrite a flac tag.
@@ -757,10 +744,11 @@ flac_tag_read_file_tag (GFile *file,
         {
             FileTag->saved = FALSE;
         }
+
+        return rc;
     }
 #endif
 
-    g_free(filename_utf8);
     return TRUE;
 }
 
@@ -819,6 +807,13 @@ flac_tag_write_file_tag (const ET_File *ETFile,
                          GError **error)
 {
     const File_Tag *FileTag;
+    GFile *file;
+    GFileIOStream *iostream;
+    EtFlacWriteState state;
+    FLAC__IOCallbacks callbacks = { et_flac_read_func, et_flac_write_func,
+                                    et_flac_seek_func, et_flac_tell_func,
+                                    et_flac_eof_func,
+                                    et_flac_write_close_func };
     const gchar *filename;
     const gchar *filename_utf8;
     const gchar *flac_error_msg;
@@ -836,31 +831,56 @@ flac_tag_write_file_tag (const ET_File *ETFile,
 
     /* libFLAC is able to detect (and skip) ID3v2 tags by itself */
     
-    // Create a new chain instance to get all blocks in one time
-    chain = FLAC__metadata_chain_new();
-    if (chain == NULL || !FLAC__metadata_chain_read(chain,filename))
+    /* Create a new chain instance to get all blocks in one time. */
+    chain = FLAC__metadata_chain_new ();
+
+    if (chain == NULL)
     {
-        if (chain == NULL)
-        {
-            // Error with "FLAC__metadata_chain_new"
-            flac_error_msg = FLAC__Metadata_ChainStatusString[FLAC__METADATA_CHAIN_STATUS_MEMORY_ALLOCATION_ERROR];
-        }else
-        {
-            // Error with "FLAC__metadata_chain_read"
-            FLAC__Metadata_ChainStatus status = FLAC__metadata_chain_status(chain);
-            flac_error_msg = FLAC__Metadata_ChainStatusString[status];
-            
-            FLAC__metadata_chain_delete(chain);
-        }
+        flac_error_msg = FLAC__Metadata_ChainStatusString[FLAC__METADATA_CHAIN_STATUS_MEMORY_ALLOCATION_ERROR];
         
         g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
                      _("Error while opening file ‘%s’ as FLAC: %s"),
                      filename_utf8, flac_error_msg);
         return FALSE;
     }
+
+    file = g_file_new_for_path (filename);
+
+    state.file = file;
+    state.error = NULL;
+    /* TODO: Fallback to an in-memory copy of the file for non-local files,
+     * where creation of the GFileIOStream may fail. */
+    iostream = g_file_open_readwrite (file, NULL, &state.error);
+
+    if (iostream == NULL)
+    {
+        FLAC__metadata_chain_delete (chain);
+        g_propagate_error (error, state.error);
+        g_object_unref (file);
+        return FALSE;
+    }
+
+    state.istream = G_FILE_INPUT_STREAM (g_io_stream_get_input_stream (G_IO_STREAM (iostream)));
+    state.ostream = G_FILE_OUTPUT_STREAM (g_io_stream_get_output_stream (G_IO_STREAM (iostream)));
+    state.seekable = G_SEEKABLE (iostream);
+    state.iostream = iostream;
+
+    if (!FLAC__metadata_chain_read_with_callbacks (chain, &state, callbacks))
+    {
+        const FLAC__Metadata_ChainStatus status = FLAC__metadata_chain_status (chain);
+        flac_error_msg = FLAC__Metadata_ChainStatusString[status];
+        FLAC__metadata_chain_delete (chain);
+
+        g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
+                     _("Error while opening file ‘%s’ as FLAC: %s"),
+                     filename_utf8, flac_error_msg);
+        et_flac_write_close_func (&state);
+        return FALSE;
+    }
     
-    // Create a new iterator instance for the chain
-    iter = FLAC__metadata_iterator_new();
+    /* Create a new iterator instance for the chain. */
+    iter = FLAC__metadata_iterator_new ();
+
     if (iter == NULL)
     {
         flac_error_msg = FLAC__Metadata_ChainStatusString[FLAC__METADATA_CHAIN_STATUS_MEMORY_ALLOCATION_ERROR];
@@ -868,19 +888,19 @@ flac_tag_write_file_tag (const ET_File *ETFile,
         g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
                      _("Error while opening file ‘%s’ as FLAC: %s"),
                      filename_utf8, flac_error_msg);
+        FLAC__metadata_chain_delete (chain);
+        et_flac_write_close_func (&state);
         return FALSE;
     }
     
-    // Initialize the iterator to point to the first metadata block in the given chain.
-    FLAC__metadata_iterator_init(iter,chain);
+    FLAC__metadata_iterator_init (iter, chain);
     
-    while (FLAC__metadata_iterator_next(iter))
+    while (FLAC__metadata_iterator_next (iter))
     {
-        //g_print("Write: %d %s -> block type: %d\n",j++,g_path_get_basename(filename),FLAC__metadata_iterator_get_block_type(iter));
-        
-        // Action to do according the type
-        switch ( FLAC__metadata_iterator_get_block_type(iter) )
+        switch (FLAC__metadata_iterator_get_block_type (iter))
         {
+            /* TODO: Modify the blocks directly, rather than deleting and
+             * recreating. */
             //
             // Delete the VORBIS_COMMENT block and convert to padding. But before, save the original vendor string.
             //
@@ -911,7 +931,7 @@ flac_tag_write_file_tag (const ET_File *ETFile,
                 FLAC__metadata_iterator_delete_block(iter,true);
                 break;
             }
-            
+
             default:
                 break;
         }
@@ -1049,14 +1069,18 @@ flac_tag_write_file_tag (const ET_File *ETFile,
      * Picture *
      ***********/
     {
-        Picture *pic = FileTag->picture;
+        EtPicture *pic = FileTag->picture;
+
         while (pic)
         {
-            if (pic->data)
+            /* TODO: Can this ever be NULL? */
+            if (pic->bytes)
             {
                 const gchar *violation;
                 FLAC__StreamMetadata *picture_block; // For picture data
                 Picture_Format format;
+                gconstpointer data;
+                gsize data_size;
                 
                 // Allocate block for picture data
                 picture_block = FLAC__metadata_object_new(FLAC__METADATA_TYPE_PICTURE);
@@ -1082,8 +1106,10 @@ flac_tag_write_file_tag (const ET_File *ETFile,
                 picture_block->data.picture.depth  = 0;
 
                 /* Picture data. */
+                data = g_bytes_get_data (pic->bytes, &data_size);
                 FLAC__metadata_object_picture_set_data (picture_block,
-                                                        (FLAC__byte *)pic->data, (FLAC__uint32) pic->size,
+                                                        (FLAC__byte *)data,
+                                                        (FLAC__uint32)data_size,
                                                         TRUE);
                 
                 if (!FLAC__metadata_object_picture_is_legal (picture_block,
@@ -1104,37 +1130,96 @@ flac_tag_write_file_tag (const ET_File *ETFile,
         }
     }
     
-    // Free iter
-    FLAC__metadata_iterator_delete(iter);
-    
+    FLAC__metadata_iterator_delete (iter);
     
     //
     // Prepare for writing tag
     //
     
-    // Move all PADDING blocks to the end on the metadata, and merge them into a single block.
-    FLAC__metadata_chain_sort_padding(chain);
+    FLAC__metadata_chain_sort_padding (chain);
  
     /* Write tag. */
-    if (!FLAC__metadata_chain_write (chain, /*padding*/TRUE,
-                                     g_settings_get_boolean (MainSettings,
-                                                             "file-preserve-modification-time")))
+    if (FLAC__metadata_chain_check_if_tempfile_needed (chain, true))
     {
-        // Error with "FLAC__metadata_chain_write"
-        FLAC__Metadata_ChainStatus status = FLAC__metadata_chain_status(chain);
-        flac_error_msg = FLAC__Metadata_ChainStatusString[status];
+        EtFlacWriteState temp_state;
+        GFile *temp_file;
+        GFileIOStream *temp_iostream;
+        GError *temp_error = NULL;
 
-        FLAC__metadata_chain_delete(chain);
-        
-        g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
-                     _("Failed to write comments to file ‘%s’: %s"),
-                     filename_utf8, flac_error_msg);
-        return FALSE;
+        temp_file = g_file_new_tmp ("easytag-XXXXXX", &temp_iostream,
+                                    &temp_error);
+
+        if (temp_file == NULL)
+        {
+            FLAC__metadata_chain_delete (chain);
+            g_propagate_error (error, temp_error);
+            et_flac_write_close_func (&state);
+            return FALSE;
+        }
+
+        temp_state.file = temp_file;
+        temp_state.error = NULL;
+        temp_state.istream = G_FILE_INPUT_STREAM (g_io_stream_get_input_stream (G_IO_STREAM (temp_iostream)));
+        temp_state.ostream = G_FILE_OUTPUT_STREAM (g_io_stream_get_output_stream (G_IO_STREAM (temp_iostream)));
+        temp_state.seekable = G_SEEKABLE (temp_iostream);
+        temp_state.iostream = temp_iostream;
+
+        if (!FLAC__metadata_chain_write_with_callbacks_and_tempfile (chain,
+                                                                     true,
+                                                                     &state,
+                                                                     callbacks,
+                                                                     &temp_state,
+                                                                     callbacks))
+        {
+            const FLAC__Metadata_ChainStatus status = FLAC__metadata_chain_status (chain);
+            flac_error_msg = FLAC__Metadata_ChainStatusString[status];
+
+            FLAC__metadata_chain_delete (chain);
+            et_flac_write_close_func (&temp_state);
+            et_flac_write_close_func (&state);
+
+            g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
+                         _("Failed to write comments to file ‘%s’: %s"),
+                         filename_utf8, flac_error_msg);
+            return FALSE;
+        }
+
+        if (!g_file_move (temp_file, file, G_FILE_COPY_OVERWRITE, NULL, NULL,
+                          NULL, &state.error))
+        {
+            FLAC__metadata_chain_delete (chain);
+            et_flac_write_close_func (&temp_state);
+
+            g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
+                         _("Failed to write comments to file ‘%s’: %s"),
+                         filename_utf8, state.error->message);
+            et_flac_write_close_func (&state);
+            return FALSE;
+        }
+
+        et_flac_write_close_func (&temp_state);
     }
-    
-    FLAC__metadata_chain_delete(chain);
+    else
+    {
+        if (!FLAC__metadata_chain_write_with_callbacks (chain, true, &state,
+                                                        callbacks))
+        {
+            const FLAC__Metadata_ChainStatus status = FLAC__metadata_chain_status (chain);
+            flac_error_msg = FLAC__Metadata_ChainStatusString[status];
+
+            FLAC__metadata_chain_delete (chain);
+            et_flac_write_close_func (&state);
+
+            g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
+                         _("Failed to write comments to file ‘%s’: %s"),
+                         filename_utf8, flac_error_msg);
+            return FALSE;
+        }
+    }
+
+    FLAC__metadata_chain_delete (chain);
+    et_flac_write_close_func (&state);
 
-    
 #ifdef ENABLE_MP3
     {
         // Delete the ID3 tags (create a dummy ETFile for the Id3tag_... function)
