@@ -19,19 +19,14 @@
 
 #include "config.h"
 
-#include <gtk/gtk.h>
 #include <glib/gi18n.h>
-#include <stdio.h>
-#include <errno.h>
 #include <string.h>
-#include <ctype.h>
-#include <unistd.h>
+#include <errno.h>
 
 #include "id3_tag.h"
 #include "ape_tag.h"
 #include "picture.h"
 #include "easytag.h"
-#include "browser.h"
 #include "genres.h"
 #include "setting.h"
 #include "misc.h"
@@ -101,7 +96,7 @@ et_id3tag_get_tpos_from_file_tag (const File_Tag *FileTag)
 
     while (*p)
     {
-        if (!isdigit (*p))
+        if (!g_ascii_isdigit (*p))
         {
             break;
         }
@@ -117,7 +112,7 @@ et_id3tag_get_tpos_from_file_tag (const File_Tag *FileTag)
 
         while (*p)
         {
-            if (!isdigit (*p))
+            if (!g_ascii_isdigit (*p))
             {
                 break;
             }
@@ -172,7 +167,7 @@ id3tag_write_file_v23tag (const ET_File *ETFile,
     ID3Field *id3_field;
     //gchar *string;
     gchar *string1;
-    Picture *pic;
+    EtPicture *pic;
 
     g_return_val_if_fail (ETFile != NULL && ETFile->FileTag != NULL, FALSE);
     g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
@@ -525,7 +520,13 @@ id3tag_write_file_v23tag (const ET_File *ETFile,
             Id3tag_Set_Field(id3_frame, ID3FN_DESCRIPTION, pic->description);
 
         if ((id3_field = ID3Frame_GetField(id3_frame,ID3FN_DATA)))
-            ID3Field_SetBINARY(id3_field, pic->data, pic->size);
+        {
+            gconstpointer data;
+            gsize data_size;
+
+            data = g_bytes_get_data (pic->bytes, &data_size);
+            ID3Field_SetBINARY (id3_field, data, data_size);
+        }
 
         has_picture = TRUE;
     }
@@ -1005,13 +1006,13 @@ gchar *Id3tag_Get_Field (const ID3Frame *id3_frame, ID3_FieldID id3_fieldid)
         switch ( enc )
         {
             case ID3TE_ISO8859_1:
-                string = g_malloc0(sizeof(char)*ID3V2_MAX_STRING_LEN+1);
+                string = g_malloc0 (ID3V2_MAX_STRING_LEN + 1);
                 num_chars = ID3Field_GetASCII_1(id3_field,string,ID3V2_MAX_STRING_LEN,0);
                 string1 = convert_string(string,"ISO-8859-1","UTF-8",FALSE);
                 break;
 
             case ID3TE_UTF8: // Shouldn't work with id3lib 3.8.3 (supports only ID3v2.3, not ID3v2.4)
-                string = g_malloc0(sizeof(char)*ID3V2_MAX_STRING_LEN+1);
+                string = g_malloc0 (ID3V2_MAX_STRING_LEN+1);
                 num_chars = ID3Field_GetASCII_1(id3_field,string,ID3V2_MAX_STRING_LEN,0);
                 //string1 = convert_string(string,"UTF-8","UTF-8",FALSE); // Nothing to do
                 if (g_utf8_validate(string,-1,NULL))
@@ -1028,7 +1029,7 @@ gchar *Id3tag_Get_Field (const ID3Frame *id3_frame, ID3_FieldID id3_fieldid)
                 break;
 
             default:
-                string = g_malloc0(sizeof(char)*4*ID3V2_MAX_STRING_LEN+1);
+                string = g_malloc0 (4 * ID3V2_MAX_STRING_LEN + 1);
                 num_chars = ID3Field_GetASCII_1(id3_field,string,ID3V2_MAX_STRING_LEN,0);
                 string1 = convert_to_utf8(string);
                 break;
