@@ -23,7 +23,6 @@
 
 #ifdef ENABLE_MP4
 
-#include <gtk/gtk.h>
 #include <glib/gi18n.h>
 
 #include "mp4_header.h"
@@ -210,15 +209,17 @@ mp4tag_read_file_tag (GFile *file,
         const TagLib::MP4::CoverArtList covers = cover.toCoverArtList ();
         const TagLib::MP4::CoverArt &art = covers.front ();
 
-        FileTag->picture = Picture_Allocate ();
+        /* TODO: Use g_bytes_new_with_free_func()? */
+        GBytes *bytes = g_bytes_new (art.data ().data (), art.data ().size ());
 
-        FileTag->picture->size = art.data ().size ();
-        FileTag->picture->data = (guchar *)g_memdup (art.data ().data (),
-                                                     art.data ().size ());
+        /* MP4 does not support image types, nor descriptions. */
+        FileTag->picture = et_picture_new (ET_PICTURE_TYPE_FRONT_COVER, "", 0,
+                                           0, bytes);
+        g_bytes_unref (bytes);
     }
     else
     {
-        FileTag->picture = NULL;
+        et_file_tag_set_picture (FileTag, NULL);
     }
 
     return TRUE;
@@ -439,6 +440,8 @@ mp4tag_write_file_tag (const ET_File *ETFile,
     {
         Picture_Format pf;
         TagLib::MP4::CoverArt::Format f;
+        gconstpointer data;
+        gsize data_size;
 
         pf = Picture_Format_From_Data (FileTag->picture);
 
@@ -459,8 +462,9 @@ mp4tag_write_file_tag (const ET_File *ETFile,
                 break;
         }
 
-        TagLib::MP4::CoverArt art (f, TagLib::ByteVector((char *)FileTag->picture->data,
-                                                         FileTag->picture->size));
+        data = g_bytes_get_data (FileTag->picture->bytes, &data_size);
+        TagLib::MP4::CoverArt art (f, TagLib::ByteVector((char *)data,
+                                                         data_size));
 
         extra_items.insert ("covr",
                             TagLib::MP4::Item (TagLib::MP4::CoverArtList ().append (art)));
